@@ -201,6 +201,19 @@ struct SetupConfig
      * @brief fsFactories Provides an external filesystem factory implementation
      */
     std::vector<std::shared_ptr<fs::IFileSystemFactory>> fsFactories;
+
+    /**
+     * @brief logLevel The default log level to initialize the medialibrary with.
+     * This can be overwriten at a later point using IMediaLibrary::setVerbosity
+     */
+    LogLevel logLevel = LogLevel::Error;
+
+    /**
+     * @brief logger An ILogger instance if the application wishes to use a
+     * custom one.
+     * If nullptr is provided, the default IOstream logger will be used.
+     */
+    std::shared_ptr<ILogger> logger;
 };
 
 class IMediaLibraryCb
@@ -222,17 +235,6 @@ public:
     virtual void onMediaModified( std::set<int64_t> mediaIds ) = 0;
 
     virtual void onMediaDeleted( std::set<int64_t> mediaIds ) = 0;
-    /**
-     * @brief onMediaConvertedToExternal Will be invoked when some media are converted
-     * from internal to external
-     * @param mediaIds The converted media IDs
-     *
-     * A media will be converted from internal to external if the entrypoint which
-     * contains it gets removed from the list of known entry points. The media
-     * will only be deleted at a later point from the database, if they haven't
-     * been played for around 6 months, and are not part of any playlist
-     */
-    virtual void onMediaConvertedToExternal( std::set<int64_t> mediaIds ) = 0;
 
     virtual void onArtistsAdded( std::vector<ArtistPtr> artists ) = 0;
     virtual void onArtistsModified( std::set<int64_t> artistsIds ) = 0;
@@ -795,7 +797,6 @@ public:
      * hierarchy isn't preserved, and the results are flattened.
      */
     virtual Query<IFolder> bannedEntryPoints() const = 0;
-    virtual void setLogger( ILogger* logger ) = 0;
     /**
      * @brief pauseBackgroundOperations Will stop potentially CPU intensive background
      * operations, until resumeBackgroundOperations() is called.
