@@ -27,11 +27,13 @@
 #include <algorithm>
 
 #include "Album.h"
-#include "AlbumTrack.h"
 #include "Artist.h"
 #include "Genre.h"
 #include "Media.h"
 #include "Thumbnail.h"
+#include "utils/Enums.h"
+#include "utils/ModificationsNotifier.h"
+#include "Deprecated.h"
 
 #include "database/SqliteTools.h"
 #include "database/SqliteQuery.h"
@@ -54,21 +56,21 @@ Album::Album(MediaLibraryPtr ml, sqlite::Row& row)
     , m_nbTracks( row.extract<decltype(m_nbTracks)>() )
     , m_duration( row.extract<decltype(m_duration)>() )
     , m_nbDiscs( row.extract<decltype(m_nbDiscs)>() )
-    , m_isPresent( row.extract<decltype(m_isPresent)>() )
+    , m_nbPresentTracks( row.extract<decltype(m_nbPresentTracks)>() )
 {
     assert( row.hasRemainingColumns() == false );
 }
 
-Album::Album( MediaLibraryPtr ml, const std::string& title )
+Album::Album( MediaLibraryPtr ml, std::string title )
     : m_ml( ml )
     , m_id( 0 )
-    , m_title( title )
+    , m_title( std::move( title ) )
     , m_artistId( 0 )
     , m_releaseYear( ~0u )
     , m_nbTracks( 0 )
     , m_duration( 0 )
     , m_nbDiscs( 1 )
-    , m_isPresent( true )
+    , m_nbPresentTracks( 0 )
 {
 }
 
@@ -80,7 +82,7 @@ Album::Album( MediaLibraryPtr ml, const Artist* artist )
     , m_nbTracks( 0 )
     , m_duration( 0 )
     , m_nbDiscs( 1 )
-    , m_isPresent( true )
+    , m_nbPresentTracks( 0 )
 {
 }
 
@@ -107,7 +109,7 @@ bool Album::setReleaseYear( unsigned int date, bool force )
         return true;
     if ( force == false )
     {
-        if ( m_releaseYear != ~0u && date != m_releaseYear )
+        if ( m_releaseYear != ~0u )
         {
             // If we already have set the date back to 0, don't do it again.
             if ( m_releaseYear == 0 )
@@ -138,11 +140,12 @@ bool Album::setShortSummary( const std::string& summary )
     return true;
 }
 
-bool Album::isThumbnailGenerated( ThumbnailSizeType sizeType ) const
+ThumbnailStatus Album::thumbnailStatus( ThumbnailSizeType sizeType ) const
 {
-    if ( m_thumbnails[Thumbnail::SizeToInt( sizeType )] != nullptr )
-        return true;
-    return thumbnail( sizeType ) != nullptr;
+    auto t = thumbnail( sizeType );
+    if ( t == nullptr )
+        return ThumbnailStatus::Missing;
+    return t->status();
 }
 
 const std::string& Album::thumbnailMrl( ThumbnailSizeType sizeType ) const
@@ -167,34 +170,26 @@ std::shared_ptr<Thumbnail> Album::thumbnail( ThumbnailSizeType sizeType ) const
     return m_thumbnails[idx];
 }
 
-bool Album::shouldUpdateThumbnail( Thumbnail& currentThumbnail,
-                                   Thumbnail::Origin newOrigin )
+bool Album::shouldUpdateThumbnail( const Thumbnail& currentThumbnail )
 {
+    /*
+     * We want to update an album thumbnail.
+     *
+     * If we inherited the thumbnail from a media, we need to insert a new record
+     * as we won't want to update the source media's thumbnail.
+     *
+     * If the cover comes from an album file, we can update it, and let the
+     * thumbnails that were based on this album be updated as well
+     *
+     * In other cases, let's just insert a new thumbnail.
+     */
     switch ( currentThumbnail.origin() )
     {
-        case Thumbnail::Origin::Artist:
-        case Thumbnail::Origin::AlbumArtist:
         case Thumbnail::Origin::Media:
-        {
-            // In all this cases, we are only interested in an album specific
-            // thumbnail, ie. provided by a cover file, or a user provided cover
-            return newOrigin == Thumbnail::Origin::CoverFile ||
-                   newOrigin == Thumbnail::Origin::UserProvided;
-        }
+            return false;
         case Thumbnail::Origin::CoverFile:
-        case Thumbnail::Origin::UserProvided:
-        {
-            // Another cover file for the same album is dubious, and is likely the
-            // same album being discovered, but with another file than when the
-            // thumbnail got originally assigned to this album. Let's reject it.
-            // And as usual, accept anything which is user provided.
-
-            // And for user provided thumbnail, only another user provided
-            // thumbnail can override it.
-            return newOrigin == Thumbnail::Origin::UserProvided;
-        }
+            return true;
         default:
-            assert( !"Unreachable" );
             return false;
     }
 }
@@ -204,24 +199,24 @@ bool Album::setThumbnail( std::shared_ptr<Thumbnail> newThumbnail )
     assert( newThumbnail != nullptr );
     auto currentThumbnail = thumbnail( newThumbnail->sizeType() );
     auto thumbnailIdx = Thumbnail::SizeToInt( newThumbnail->sizeType() );
-    if ( currentThumbnail != nullptr &&
-         shouldUpdateThumbnail( *currentThumbnail, newThumbnail->origin() ) == false )
-            return true;
-
     currentThumbnail = Thumbnail::updateOrReplace( m_ml, currentThumbnail,
-                                                   newThumbnail, m_id,
-                                                   Thumbnail::EntityType::Album );
-    auto res = currentThumbnail != nullptr;
+                                                   std::move( newThumbnail ),
+                                                   Album::shouldUpdateThumbnail,
+                                                   m_id, Thumbnail::EntityType::Album );
+    if ( currentThumbnail == nullptr )
+        return false;
     m_thumbnails[thumbnailIdx] = std::move( currentThumbnail );
-    return res;
+    auto notifier = m_ml->getNotifier();
+    if ( notifier != nullptr )
+        notifier->notifyAlbumModification( m_id );
+    return true;
 }
 
 std::string Album::addRequestJoin( const QueryParameters* params,
-        bool albumTrack )
+        bool media  )
 {
     auto sort = params != nullptr ? params->sort : SortingCriteria::Alpha;
     bool artist = false;
-    bool media = false;
 
     switch( sort )
     {
@@ -231,7 +226,7 @@ std::string Album::addRequestJoin( const QueryParameters* params,
             /* No other tables required for this criterias */
             break;
         case SortingCriteria::PlayCount:
-            albumTrack = true;
+        case SortingCriteria::InsertionDate:
             media = true;
             break;
         case SortingCriteria::Artist:
@@ -248,20 +243,15 @@ std::string Album::addRequestJoin( const QueryParameters* params,
     std::string req;
     if ( artist == true )
         req += "LEFT JOIN " + Artist::Table::Name + " art ON alb.artist_id = art.id_artist ";
-    if ( albumTrack == true )
-        req += "INNER JOIN " + AlbumTrack::Table::Name + " att ON alb.id_album = att.album_id ";
     if ( media == true )
-    {
-        assert( albumTrack == true );
-        req += "INNER JOIN " + Media::Table::Name + " m ON att.media_id = m.id_media ";
-    }
+        req += "INNER JOIN " + Media::Table::Name + " m ON m.album_id = alb.id_album ";
     return req;
 }
 
 std::string Album::orderTracksBy( const QueryParameters* params = nullptr )
 {
     std::string req = " ORDER BY ";
-    auto sort = params != nullptr ? params->sort : SortingCriteria::Default;
+    auto sort = params != nullptr ? params->sort : SortingCriteria::TrackId;
     auto desc = params != nullptr ? params->desc : false;
     switch ( sort )
     {
@@ -277,11 +267,12 @@ std::string Album::orderTracksBy( const QueryParameters* params = nullptr )
     default:
         LOG_WARN( "Unsupported sorting criteria, falling back to SortingCriteria::Default" );
         /* fall-through */
+    case SortingCriteria::TrackId:
     case SortingCriteria::Default:
         if ( desc == true )
-            req += "att.disc_number DESC, att.track_number DESC, med.filename";
+            req += "med.disc_number DESC, med.track_number DESC, med.filename";
         else
-            req += "att.disc_number, att.track_number, med.filename";
+            req += "med.disc_number, med.track_number, med.filename";
         break;
     }
 
@@ -328,6 +319,12 @@ std::string Album::orderBy( const QueryParameters* params )
             req += "DESC "; // Most played first by default
         req += ", alb.title";
         break;
+    case SortingCriteria::InsertionDate:
+        req = "GROUP BY alb.id_album "
+              "ORDER BY MIN(m.insertion_date) ";
+        if ( desc == true )
+            req += "DESC ";
+        break;
     default:
         LOG_WARN( "Unsupported sorting criteria, falling back to SortingCriteria::Default (Alpha)" );
         /* fall-through */
@@ -349,8 +346,9 @@ Query<IMedia> Album::tracks( const QueryParameters* params ) const
     // This doesn't return the cached version, because it would be fairly complicated, if not impossible or
     // counter productive, to maintain a cache that respects all orderings.
     std::string req = "FROM " + Media::Table::Name + " med "
-        " INNER JOIN " + AlbumTrack::Table::Name + " att ON att.media_id = med.id_media "
-        " WHERE att.album_id = ? AND med.is_present != 0";
+        " WHERE med.album_id = ?";
+    if ( params == nullptr || params->includeMissing == false )
+        req += " AND med.is_present != 0";
     return make_query<Media, IMedia>( m_ml, "med.*", std::move( req ),
                                       orderTracksBy( params ), m_id );
 }
@@ -360,16 +358,17 @@ Query<IMedia> Album::tracks( GenrePtr genre, const QueryParameters* params ) con
     if ( genre == nullptr )
         return {};
     std::string req = "FROM " + Media::Table::Name + " med "
-            " INNER JOIN " + AlbumTrack::Table::Name + " att ON att.media_id = med.id_media "
-            " WHERE att.album_id = ? AND med.is_present != 0"
-            " AND genre_id = ?";
+            " WHERE med.album_id = ?"
+            " AND med.genre_id = ?";
+    if ( params == nullptr || params->includeMissing == false )
+        req += " AND med.is_present != 0";
     return make_query<Media, IMedia>( m_ml, "med.*", std::move( req ),
                                       orderTracksBy( params ), m_id, genre->id() );
 }
 
 std::vector<MediaPtr> Album::cachedTracks() const
 {
-    if ( m_tracks.size() == 0 )
+    if ( m_tracks.empty() == true )
         m_tracks = tracks( nullptr )->all();
     return m_tracks;
 }
@@ -380,20 +379,19 @@ Query<IMedia> Album::searchTracks( const std::string& pattern,
     return Media::searchAlbumTracks( m_ml, pattern, m_id, params );
 }
 
-std::shared_ptr<AlbumTrack> Album::addTrack( std::shared_ptr<Media> media, unsigned int trackNb,
-                                             unsigned int discNumber, int64_t artistId, Genre* genre )
+bool Album::addTrack( std::shared_ptr<Media> media, unsigned int trackNb,
+                      unsigned int discNumber, int64_t artistId, Genre* genre )
 {
-    auto track = AlbumTrack::create( m_ml, m_id, media->id(), trackNb, discNumber, artistId,
-                                     genre != nullptr ? genre->id() : 0, media->duration() );
-    if ( track == nullptr )
-        return nullptr;
-    media->setAlbumTrack( track );
-    if ( genre != nullptr )
-        genre->updateCachedNbTracks( 1 );
-    // Assume the media will be saved by the caller
+    auto duration = media->duration() >= 0 ? media->duration() : 0;
+    if ( media->markAsAlbumTrack( m_id, trackNb, discNumber, artistId, genre ) == false )
+        return false;
+    if ( genre != nullptr && genre->updateNbTracks( 1 ) == false )
+        return false;
+
     m_nbTracks++;
-    if ( media->duration() > 0 )
-        m_duration += media->duration();
+    assert( media->isPresent() );
+    ++m_nbPresentTracks;
+    m_duration += duration;
     // Don't assume we have always have a valid value in m_tracks.
     // While it's ok to assume that if we are currently parsing the album, we
     // have a valid cache tracks, this isn't true when restarting an interrupted parsing.
@@ -402,17 +400,36 @@ std::shared_ptr<AlbumTrack> Album::addTrack( std::shared_ptr<Media> media, unsig
     if ( ( m_tracks.empty() == true && m_nbTracks == 1 ) ||
          ( m_tracks.empty() == false && m_nbTracks > 1 ) )
         m_tracks.push_back( std::move( media ) );
-    return track;
+    return true;
 }
 
-bool Album::removeTrack( Media& media, AlbumTrack& track )
+bool Album::removeTrack( Media& media )
 {
-    m_duration -= media.duration();
+    /*
+     * Remove references to genre/album/artist from the media table before we
+     * start updating album & genre.
+     * If we don't do this first, if we're removing the last album track we'd
+     * end up with a genre/album being deleted while there's still a foreign key
+     * pointing to it
+     */
+    if ( media.setSubTypeUnknown() == false )
+        return false;
+
+    static const std::string req = "UPDATE " + Table::Name + " SET "
+        "duration = duration - ? "
+        "WHERE id_album = ?";
+    auto duration = media.duration() >= 0 ? media.duration() : 0;
+    if ( sqlite::Tools::executeUpdate( m_ml->getConn(), req,
+                                       duration, m_id ) == false )
+        return false;
+
+    auto genre = std::static_pointer_cast<Genre>( media.genre() );
+    if ( genre != nullptr && genre->updateNbTracks( -1 ) )
+        return false;
+
+    m_duration -= duration;
     m_nbTracks--;
-    auto genre = std::static_pointer_cast<Genre>( track.genre() );
-    if ( genre != nullptr )
-        genre->updateCachedNbTracks( -1 );
-    auto it = std::find_if( begin( m_tracks ), end( m_tracks ), [&media]( MediaPtr m ) {
+    auto it = std::find_if( begin( m_tracks ), end( m_tracks ), [&media]( MediaPtr& m ) {
         return m->id() == media.id();
     });
     if ( it != end( m_tracks ) )
@@ -426,6 +443,11 @@ unsigned int Album::nbTracks() const
     return m_nbTracks;
 }
 
+uint32_t Album::nbPresentTracks() const
+{
+    return m_nbPresentTracks;
+}
+
 uint32_t Album::nbDiscs() const
 {
     return m_nbDiscs;
@@ -470,26 +492,20 @@ bool Album::setAlbumArtist( std::shared_ptr<Artist> artist )
             "artist_id = ? WHERE id_album = ?";
     if ( sqlite::Tools::executeUpdate( m_ml->getConn(), req, artist->id(), m_id ) == false )
         return false;
-    if ( m_artistId != 0 )
-    {
-        if ( m_albumArtist == nullptr )
-            albumArtist();
-        m_albumArtist->updateNbAlbum( -1 );
-    }
     m_artistId = artist->id();
-    m_albumArtist = artist;
-    artist->updateNbAlbum( 1 );
+    m_albumArtist = std::move( artist );
     static const std::string ftsReq = "UPDATE " + FtsTable::Name + " SET "
             " artist = ? WHERE rowid = ?";
-    return sqlite::Tools::executeUpdate( m_ml->getConn(), ftsReq, artist->name(), m_id );
+    return sqlite::Tools::executeUpdate( m_ml->getConn(), ftsReq,
+                                         m_albumArtist->name(), m_id );
 }
 
 Query<IArtist> Album::artists( const QueryParameters* params ) const
 {
     std::string req = "FROM " + Artist::Table::Name + " art "
-            "INNER JOIN " + AlbumTrack::Table::Name + " att "
-                "ON att.artist_id = art.id_artist "
-            "WHERE att.album_id = ?";
+            "INNER JOIN " + Media::Table::Name + " m "
+                "ON m.artist_id = art.id_artist "
+            "WHERE m.album_id = ?";
     if ( params != nullptr && ( params->sort != SortingCriteria::Alpha &&
                                 params->sort != SortingCriteria::Default ) )
         LOG_WARN( "Unsupported sorting criteria, falling back to SortingCriteria::Alpha" );
@@ -512,61 +528,27 @@ void Album::createTable( sqlite::Connection* dbConnection )
 
 void Album::createTriggers( sqlite::Connection* dbConnection )
 {
-    const std::string indexReq = "CREATE INDEX IF NOT EXISTS album_artist_id_idx ON " +
-            Table::Name + "(artist_id)";
-    static const std::string triggerReq = "CREATE TRIGGER IF NOT EXISTS is_album_present AFTER UPDATE OF "
-            "is_present ON " + Media::Table::Name +
-            " WHEN new.subtype = " +
-                std::to_string( static_cast<typename std::underlying_type<IMedia::SubType>::type>(
-                                    IMedia::SubType::AlbumTrack ) ) +
-            " BEGIN "
-            " UPDATE " + Table::Name + " SET is_present=is_present + "
-                "(CASE new.is_present WHEN 0 THEN -1 ELSE 1 END)"
-                "WHERE id_album = (SELECT album_id FROM " + AlbumTrack::Table::Name + " "
-                    "WHERE media_id = new.id_media"
-                ");"
-            " END";
-    static const std::string deleteTriggerReq = "CREATE TRIGGER IF NOT EXISTS delete_album_track AFTER DELETE ON "
-             + AlbumTrack::Table::Name +
-            " BEGIN "
-            " UPDATE " + Table::Name +
-            " SET"
-                " nb_tracks = nb_tracks - 1,"
-                " is_present = is_present - 1,"
-                " duration = duration - old.duration"
-                " WHERE id_album = old.album_id;"
-            " DELETE FROM " + Table::Name +
-                " WHERE id_album=old.album_id AND nb_tracks = 0;"
-            " END";
-    static const std::string updateAddTrackTriggerReq = "CREATE TRIGGER IF NOT EXISTS add_album_track"
-            " AFTER INSERT ON " + AlbumTrack::Table::Name +
-            " BEGIN"
-            " UPDATE " + Table::Name +
-            " SET duration = duration + new.duration,"
-            " nb_tracks = nb_tracks + 1,"
-            " is_present = is_present + 1"
-            " WHERE id_album = new.album_id;"
-            " END";
-    static const std::string vtriggerInsert = "CREATE TRIGGER IF NOT EXISTS insert_album_fts AFTER INSERT ON "
-            + Table::Name +
-            // Skip unknown albums
-            " WHEN new.title IS NOT NULL"
-            " BEGIN"
-            " INSERT INTO " + FtsTable::Name + "(rowid, title) VALUES(new.id_album, new.title);"
-            " END";
-    static const std::string vtriggerDelete = "CREATE TRIGGER IF NOT EXISTS delete_album_fts BEFORE DELETE ON "
-            + Table::Name +
-            // Unknown album probably won't be deleted, but better safe than sorry
-            " WHEN old.title IS NOT NULL"
-            " BEGIN"
-            " DELETE FROM " + FtsTable::Name + " WHERE rowid = old.id_album;"
-            " END";
-    sqlite::Tools::executeRequest( dbConnection, indexReq );
-    sqlite::Tools::executeRequest( dbConnection, triggerReq );
-    sqlite::Tools::executeRequest( dbConnection, deleteTriggerReq );
-    sqlite::Tools::executeRequest( dbConnection, updateAddTrackTriggerReq );
-    sqlite::Tools::executeRequest( dbConnection, vtriggerInsert );
-    sqlite::Tools::executeRequest( dbConnection, vtriggerDelete );
+    sqlite::Tools::executeRequest( dbConnection,
+                                   trigger( Triggers::IsPresent, Settings::DbModelVersion ) );
+    sqlite::Tools::executeRequest( dbConnection,
+                                   trigger( Triggers::DeleteTrack, Settings::DbModelVersion ) );
+    sqlite::Tools::executeRequest( dbConnection,
+                                   trigger( Triggers::InsertFts, Settings::DbModelVersion ) );
+    sqlite::Tools::executeRequest( dbConnection,
+                                   trigger( Triggers::DeleteFts, Settings::DbModelVersion ) );
+    sqlite::Tools::executeRequest( dbConnection,
+                                   trigger( Triggers::DeleteEmpty, Settings::DbModelVersion ) );
+    sqlite::Tools::executeRequest( dbConnection,
+                                   trigger( Triggers::UpdateOnMediaAlbumIdChange,
+                                            Settings::DbModelVersion ) );
+}
+
+void Album::createIndexes( sqlite::Connection* dbConnection )
+{
+    sqlite::Tools::executeRequest( dbConnection,
+                                   index( Indexes::ArtistId, Settings::DbModelVersion ) );
+    sqlite::Tools::executeRequest( dbConnection,
+                                   index( Indexes::NbTracks, Settings::DbModelVersion ) );
 }
 
 std::string Album::schema( const std::string& tableName, uint32_t dbModel )
@@ -627,22 +609,253 @@ std::string Album::schema( const std::string& tableName, uint32_t dbModel )
     return "<not a valid request>";
 }
 
-bool Album::checkDbModel( MediaLibraryPtr ml )
+std::string Album::trigger( Triggers trigger, uint32_t dbModel )
 {
-    return sqlite::Tools::checkSchema( ml->getConn(),
-                                       schema( Table::Name, Settings::DbModelVersion ),
-                                       Table::Name ) &&
-           sqlite::Tools::checkSchema( ml->getConn(),
-                                       schema( FtsTable::Name, Settings::DbModelVersion ),
-                                       FtsTable::Name );
+    switch ( trigger )
+    {
+        case Triggers::IsPresent:
+        {
+            if ( dbModel < 23 )
+            {
+                return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                        " AFTER UPDATE OF is_present ON " + Media::Table::Name +
+                        " WHEN new.subtype = " +
+                            utils::enum_to_string( IMedia::SubType::AlbumTrack ) +
+                        " BEGIN "
+                        " UPDATE " + Table::Name + " SET is_present=is_present + "
+                            "(CASE new.is_present WHEN 0 THEN -1 ELSE 1 END)"
+                            "WHERE id_album = (SELECT album_id FROM " + AlbumTrack::Table::Name + " "
+                                "WHERE media_id = new.id_media"
+                            ");"
+                        " END";
+            }
+            else if ( dbModel < 34 )
+            {
+                return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                       " AFTER UPDATE OF is_present ON " + Media::Table::Name +
+                       " WHEN new.subtype = " +
+                           utils::enum_to_string( IMedia::SubType::AlbumTrack ) +
+                       " AND old.is_present != new.is_present"
+                       " BEGIN "
+                       " UPDATE " + Table::Name + " SET is_present=is_present + "
+                           "(CASE new.is_present WHEN 0 THEN -1 ELSE 1 END)"
+                           "WHERE id_album = (SELECT album_id FROM " + AlbumTrack::Table::Name + " "
+                               "WHERE media_id = new.id_media"
+                           ");"
+                       " END";
+            }
+            return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                   " AFTER UPDATE OF is_present ON " + Media::Table::Name +
+                   " WHEN new.subtype = " +
+                       utils::enum_to_string( IMedia::SubType::AlbumTrack ) +
+                   " AND old.is_present != new.is_present"
+                   " BEGIN "
+                   " UPDATE " + Table::Name + " SET is_present=is_present + "
+                       "(CASE new.is_present WHEN 0 THEN -1 ELSE 1 END)"
+                       "WHERE id_album = new.album_id;"
+                   " END";
+        }
+        case Triggers::AddTrack:
+        {
+            assert( dbModel < 34 );
+            return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                   " AFTER INSERT ON " + AlbumTrack::Table::Name +
+                   " BEGIN"
+                   " UPDATE " + Table::Name +
+                        " SET duration = duration + new.duration,"
+                        " nb_tracks = nb_tracks + 1,"
+                        " is_present = is_present + 1"
+                        " WHERE id_album = new.album_id;"
+                   " END";
+        }
+        case Triggers::DeleteTrack:
+        {
+            if ( dbModel < 34 )
+            {
+                return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                            " AFTER DELETE ON " + AlbumTrack::Table::Name +
+                       " BEGIN "
+                       " UPDATE " + Table::Name +
+                            " SET"
+                                " nb_tracks = nb_tracks - 1,"
+                                " is_present = is_present - 1,"
+                                " duration = duration - old.duration"
+                                " WHERE id_album = old.album_id;"
+                            " DELETE FROM " + Table::Name +
+                                " WHERE id_album=old.album_id AND nb_tracks = 0;"
+                       " END";
+            }
+            return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                        " AFTER DELETE ON " + Media::Table::Name +
+                    " WHEN old.subtype = " +
+                        utils::enum_to_string( IMedia::SubType::AlbumTrack ) +
+                   " BEGIN "
+                   " UPDATE " + Table::Name +
+                        " SET"
+                            " nb_tracks = nb_tracks - 1,"
+                            " is_present = is_present - IIF(old.is_present != 0, 1, 0),"
+                            " duration = duration - MAX(old.duration, 0)"
+                            " WHERE id_album = old.album_id;"
+                   " END";
+        }
+        case Triggers::InsertFts:
+        {
+            return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                    " AFTER INSERT ON " + Table::Name +
+                    // Skip unknown albums
+                    " WHEN new.title IS NOT NULL"
+                    " BEGIN"
+                        " INSERT INTO " + FtsTable::Name + "(rowid, title)"
+                            " VALUES(new.id_album, new.title);"
+                    " END";
+        }
+        case Triggers::DeleteFts:
+        {
+            return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                    " BEFORE DELETE ON " + Table::Name +
+                    // Unknown album probably won't be deleted, but better safe than sorry
+                    " WHEN old.title IS NOT NULL"
+                    " BEGIN"
+                        " DELETE FROM " + FtsTable::Name +
+                            " WHERE rowid = old.id_album;"
+                    " END";
+        }
+        case Triggers::DeleteEmpty:
+        {
+            assert( dbModel >= 34 );
+            return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                       " AFTER UPDATE OF nb_tracks ON " + Table::Name +
+                   " WHEN new.nb_tracks = 0"
+                   " BEGIN "
+                        " DELETE FROM " + Table::Name +
+                            " WHERE id_album=new.id_album;"
+                   " END";
+        }
+        case Triggers::UpdateOnMediaAlbumIdChange:
+        {
+            assert( dbModel >= 36 );
+            return "CREATE TRIGGER " + triggerName( trigger, dbModel ) +
+                       " AFTER UPDATE OF album_id ON " + Media::Table::Name +
+                       " WHEN IFNULL(old.album_id, 0) != IFNULL(new.album_id, 0)"
+                       " BEGIN"
+                           " UPDATE " + Table::Name + " SET "
+                               " is_present = is_present - IIF(old.is_present != 0, 1, 0),"
+                               " nb_tracks = nb_tracks - 1,"
+                               " duration = duration - IIF(old.duration >= 0, old.duration, 0)"
+                               " WHERE old.album_id IS NOT NULL AND id_album = old.album_id;"
+                           " UPDATE " + Table::Name + " SET "
+                               " is_present = is_present + IIF(old.is_present != 0, 1, 0),"
+                               " nb_tracks = nb_tracks + 1,"
+                               " duration = duration + IIF(new.duration >= 0, new.duration, 0)"
+                               " WHERE new.album_id IS NOT NULL AND id_album = new.album_id;"
+                       " END";
+        }
+        default:
+            assert( !"Invalid trigger provided" );
+    }
+    return "<Invalid request provided>";
 }
 
-std::shared_ptr<Album> Album::create( MediaLibraryPtr ml, const std::string& title )
+std::string Album::triggerName( Album::Triggers trigger, uint32_t dbModel )
 {
-    auto album = std::make_shared<Album>( ml, title );
+    switch ( trigger )
+    {
+        case Triggers::IsPresent:
+        {
+            if ( dbModel < 23 )
+                return "is_album_present";
+            return "album_is_present";
+        }
+        case Triggers::AddTrack:
+            assert( dbModel < 34 );
+            return "add_album_track";
+        case Triggers::DeleteTrack:
+            if ( dbModel < 34 )
+                return "delete_album_track";
+            return "album_delete_track";
+        case Triggers::InsertFts:
+            return "insert_album_fts";
+        case Triggers::DeleteFts:
+            return "delete_album_fts";
+        case Triggers::DeleteEmpty:
+            assert( dbModel >= 34 );
+            return "album_delete_empty";
+        case Triggers::UpdateOnMediaAlbumIdChange:
+            assert( dbModel >= 36 );
+            return "album_update_on_media_album_id";
+        default:
+            assert( !"Invalid trigger provided" );
+    }
+    return "<invalid request>";
+}
+
+std::string Album::index( Indexes index, uint32_t dbModel )
+{
+    switch ( index )
+    {
+        case Indexes::ArtistId:
+            return "CREATE INDEX " + indexName( index, dbModel ) + " ON " +
+                    Table::Name + "(artist_id)";
+        case Indexes::NbTracks:
+            assert( dbModel >= 34 );
+            return "CREATE INDEX " + indexName( index, dbModel ) + " ON " +
+                    Table::Name + "(nb_tracks, is_present)";
+    }
+    return "<invalid request>";
+}
+
+std::string Album::indexName( Album::Indexes index, uint32_t dbModel )
+{
+    UNUSED_IN_RELEASE( index );
+    UNUSED_IN_RELEASE( dbModel );
+
+    switch ( index )
+    {
+        case Indexes::ArtistId:
+            return "album_artist_id_idx";
+        case Indexes::NbTracks:
+            assert( dbModel >= 34 );
+            return "album_nb_tracks_idx";
+    }
+    return "<invalid request>";
+}
+
+bool Album::checkDbModel( MediaLibraryPtr ml )
+{
+    OPEN_READ_CONTEXT( ctx, ml->getConn() );
+
+    if ( sqlite::Tools::checkTableSchema(
+                                       schema( Table::Name, Settings::DbModelVersion ),
+                                       Table::Name ) == false ||
+           sqlite::Tools::checkTableSchema(
+                                       schema( FtsTable::Name, Settings::DbModelVersion ),
+                                       FtsTable::Name ) == false )
+        return false;
+
+    if ( sqlite::Tools::checkIndexStatement(
+            index( Indexes::ArtistId, Settings::DbModelVersion ),
+            indexName( Indexes::ArtistId, Settings::DbModelVersion ) ) == false )
+        return false;
+
+    auto check = []( Triggers t ) {
+        return sqlite::Tools::checkTriggerStatement(
+                                    trigger( t, Settings::DbModelVersion ),
+                                    triggerName( t, Settings::DbModelVersion ) );
+    };
+    return check( Triggers::IsPresent ) &&
+            check( Triggers::DeleteTrack ) &&
+            check( Triggers::InsertFts ) &&
+            check( Triggers::DeleteFts ) &&
+            check( Triggers::DeleteEmpty ) &&
+            check( Triggers::UpdateOnMediaAlbumIdChange );
+}
+
+std::shared_ptr<Album> Album::create( MediaLibraryPtr ml, std::string title )
+{
+    auto album = std::make_shared<Album>( ml, std::move( title ) );
     static const std::string req = "INSERT INTO " + Table::Name +
             "(id_album, title) VALUES(NULL, ?)";
-    if ( insert( ml, album, req, title ) == false )
+    if ( insert( ml, album, req, album->m_title ) == false )
         return nullptr;
     return album;
 }
@@ -664,8 +877,9 @@ Query<IAlbum> Album::search( MediaLibraryPtr ml, const std::string& pattern,
     req += addRequestJoin( params, false );
     req += "WHERE id_album IN "
             "(SELECT rowid FROM " + FtsTable::Name + " WHERE " +
-            FtsTable::Name + " MATCH ?)"
-            "AND alb.is_present != 0";
+            FtsTable::Name + " MATCH ?)";
+    if ( params == nullptr || params->includeMissing == false )
+        req += " AND alb.is_present != 0";
     return make_query<Album, IAlbum>( ml, "alb.*", std::move( req ),
                                       orderBy( params ),
                                       sqlite::Tools::sanitizePattern( pattern ) );
@@ -679,8 +893,9 @@ Query<IAlbum> Album::searchFromArtist( MediaLibraryPtr ml, const std::string& pa
     req += "WHERE id_album IN "
             "(SELECT rowid FROM " + FtsTable::Name + " WHERE " +
             FtsTable::Name + " MATCH ?)"
-            "AND alb.is_present != 0 "
-            "AND artist_id = ?";
+            " AND artist_id = ?";
+    if ( params == nullptr || params->includeMissing == false )
+        req += " AND alb.is_present != 0";
     return make_query<Album, IAlbum>( ml, "alb.*", std::move( req ),
                                       orderBy( params ),
                                       sqlite::Tools::sanitizePattern( pattern ),
@@ -690,13 +905,12 @@ Query<IAlbum> Album::searchFromArtist( MediaLibraryPtr ml, const std::string& pa
 Query<IAlbum> Album::fromArtist( MediaLibraryPtr ml, int64_t artistId, const QueryParameters* params )
 {
     std::string req = "FROM " + Table::Name + " alb "
-                    "INNER JOIN " + AlbumTrack::Table::Name + " att "
-                        "ON att.album_id = alb.id_album "
                     "INNER JOIN " + Media::Table::Name + " m "
-                        "ON att.media_id = m.id_media "
-                    "WHERE (att.artist_id = ? OR alb.artist_id = ?) "
-                        "AND m.is_present != 0 ";
-    std::string groupAndOrder = "GROUP BY att.album_id ORDER BY ";
+                        "ON m.album_id = alb.id_album "
+                    "WHERE (m.artist_id = ? OR alb.artist_id = ?)";
+    if ( params == nullptr || params->includeMissing == false )
+        req += " AND m.is_present != 0";
+    std::string groupAndOrder = " GROUP BY m.album_id ORDER BY ";
     auto sort = params != nullptr ? params->sort : SortingCriteria::Default;
     auto desc = params != nullptr ? params->desc : false;
     switch ( sort )
@@ -731,8 +945,8 @@ Query<IAlbum> Album::fromGenre( MediaLibraryPtr ml, int64_t genreId, const Query
 {
     std::string req = "FROM " + Table::Name + " alb ";
     req += addRequestJoin( params, true );
-    req += "WHERE att.genre_id = ?";
-    std::string groupAndOrderBy = "GROUP BY att.album_id" + orderBy( params );
+    req += "WHERE m.genre_id = ?";
+    std::string groupAndOrderBy = "GROUP BY m.album_id" + orderBy( params );
     return make_query<Album, IAlbum>( ml, "alb.*", std::move( req ),
                                       std::move( groupAndOrderBy ), genreId );
 }
@@ -745,8 +959,8 @@ Query<IAlbum> Album::searchFromGenre( MediaLibraryPtr ml, const std::string& pat
     req += "WHERE id_album IN "
             "(SELECT rowid FROM " + FtsTable::Name + " WHERE " +
             FtsTable::Name + " MATCH ?)"
-            "AND att.genre_id = ?";
-    std::string groupAndOrderBy = "GROUP BY att.album_id" + orderBy( params );
+            "AND m.genre_id = ?";
+    std::string groupAndOrderBy = "GROUP BY m.album_id" + orderBy( params );
     return make_query<Album, IAlbum>( ml, "alb.*", std::move( req ),
                                       std::move( groupAndOrderBy ),
                                       sqlite::Tools::sanitizePattern( pattern ),
@@ -755,11 +969,14 @@ Query<IAlbum> Album::searchFromGenre( MediaLibraryPtr ml, const std::string& pat
 
 Query<IAlbum> Album::listAll( MediaLibraryPtr ml, const QueryParameters* params )
 {
-    std::string countReq = "SELECT COUNT(*) FROM " + Table::Name + " WHERE "
-            "is_present != 0";
+    std::string countReq = "SELECT COUNT(*) FROM " + Table::Name;
     std::string req = "SELECT alb.* FROM " + Table::Name + " alb ";
     req += addRequestJoin( params, false );
-    req += "WHERE alb.is_present != 0 ";
+    if ( params == nullptr || params->includeMissing == false )
+    {
+        countReq += " WHERE is_present != 0";
+        req += "WHERE alb.is_present != 0 ";
+    }
     req += orderBy( params );
     return make_query_with_count<Album, IAlbum>( ml, std::move( countReq ),
                                                  std::move( req ) );
@@ -767,7 +984,8 @@ Query<IAlbum> Album::listAll( MediaLibraryPtr ml, const QueryParameters* params
 
 bool Album::checkDBConsistency( MediaLibraryPtr ml )
 {
-    sqlite::Statement stmt{ ml->getConn()->handle(),
+    OPEN_READ_CONTEXT( ctx, ml->getConn() );
+    sqlite::Statement stmt{
                 "SELECT nb_tracks, is_present FROM " +
                     Album::Table::Name
     };
