@@ -27,11 +27,16 @@
 #include "MediaLibrary.h"
 #include "logging/Logger.h"
 
-extern "C" medialibrary::IMediaLibrary* NewMediaLibrary()
+extern "C"
+medialibrary::IMediaLibrary* NewMediaLibrary( const char* dbPath, const char* mlFolderPath,
+                                              bool lockFile, const medialibrary::SetupConfig* cfg )
 {
     try
     {
-        return new medialibrary::MediaLibrary();
+        auto ml = medialibrary::MediaLibrary::create( dbPath, mlFolderPath, lockFile, cfg );
+        if ( ml )
+            return ml.release();
+        return nullptr;
     }
     catch ( const std::exception& ex )
     {
