@@ -392,8 +392,8 @@ Media::setLastPositionAndTime( PositionTypes positionType, float lastPos,
     m_lastPlayedDate = lastPlayedDate;
     m_lastPosition = curatedPosition;
     m_lastTime = curatedTime;
-    m_ml->getCb()->onHistoryChanged( isStream() ? HistoryType::Media :
-                                                  HistoryType::Network );
+    m_ml->getCb()->onHistoryChanged( isStream() ? HistoryType::Network :
+                                                  HistoryType::Media );
     switch ( positionType )
     {
     case PositionTypes::Begin:
@@ -452,6 +452,8 @@ bool Media::markAsPlayed()
         return false;
     m_lastPlayedDate = lastPlayedDate;
     ++m_playCount;
+    m_ml->getCb()->onHistoryChanged( isStream() ? HistoryType::Network :
+                                                  HistoryType::Media );
     return true;
 }
 
@@ -484,8 +486,7 @@ bool Media::removeFromHistory()
     m_lastTime = -1;
     m_lastPlayedDate = 0;
     m_playCount = 0;
-    auto historyType = ( m_type == Type::Video || m_type == Type::Audio ) ?
-                       HistoryType::Media : HistoryType::Network;
+    auto historyType = isStream() ? HistoryType::Network: HistoryType::Media;
     m_ml->getCb()->onHistoryChanged( historyType );
     return true;
 }
@@ -1134,11 +1135,11 @@ bool Media::setLyrics( std::string lyrics )
     return true;
 }
 
-std::string Media::addRequestJoin( const QueryParameters* params, bool forceFile )
+std::string Media::addRequestJoin( const QueryParameters* params )
 {
     bool artist = false;
     bool album = false;
-    bool file = forceFile;
+    bool file = false;
     auto sort = params != nullptr ? params->sort : SortingCriteria::Alpha;
 
     switch( sort )
@@ -1259,7 +1260,7 @@ Query<IMedia> Media::listAll( MediaLibraryPtr ml, IMedia::Type type,
     assert( type == IMedia::Type::Audio || type == IMedia::Type::Video );
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, true );
+    req += addRequestJoin( params );
     // We want to include unknown media to the video listing, so we invert the
     // filter to exclude Audio (ie. we include Unknown & Video)
     // If we only want audio media, then we just filter 'type = Audio'
@@ -1267,8 +1268,7 @@ Query<IMedia> Media::listAll( MediaLibraryPtr ml, IMedia::Type type,
         req += " WHERE m.type != ?";
     else
         req += " WHERE m.type = ?";
-    req +=  " AND (f.type = ? OR f.type = ?)"
-            " AND f.is_external = 0";
+    req +=  " AND m.import_type = ?";
     if ( params == nullptr || params->includeMissing == false )
         req += " AND m.is_present != 0";
 
@@ -1277,12 +1277,12 @@ Query<IMedia> Media::listAll( MediaLibraryPtr ml, IMedia::Type type,
         req += " AND m.subtype = ?";
         return make_query<Media, IMedia>( ml, "m.*", std::move( req ),
                                         sortRequest( params ), IMedia::Type::Audio,
-                                        IFile::Type::Main, IFile::Type::Disc, subType );
+                                        ImportType::Internal, subType );
     }
 
     return make_query<Media, IMedia>( ml, "m.*", std::move( req ),
                                       sortRequest( params ), IMedia::Type::Audio,
-                                      IFile::Type::Main, IFile::Type::Disc );
+                                      ImportType::Internal );
 }
 
 Query<IMedia> Media::listInProgress( MediaLibraryPtr ml, IMedia::Type type,
@@ -1290,7 +1290,7 @@ Query<IMedia> Media::listInProgress( MediaLibraryPtr ml, IMedia::Type type,
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, false );
+    req += addRequestJoin( params );
     req += " WHERE (m.last_position >= 0 OR m.last_time > 0)";
     if ( params == nullptr || params->includeMissing == false )
         req += " AND m.is_present != 0";
@@ -2190,7 +2190,7 @@ Query<IMedia> Media::search( MediaLibraryPtr ml, const std::string& title,
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, false );
+    req += addRequestJoin( params );
 
     req +=  " WHERE"
             " m.id_media IN (SELECT rowid FROM " + Media::FtsTable::Name +
@@ -2209,11 +2209,10 @@ Query<IMedia> Media::search( MediaLibraryPtr ml, const std::string& title,
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, true );
+    req += addRequestJoin( params );
     req +=  " WHERE"
             " m.id_media IN (SELECT rowid FROM " + Media::FtsTable::Name +
             " WHERE " + Media::FtsTable::Name + " MATCH ?)"
-            " AND (f.type = ? OR f.type = ?)"
             " AND m.type = ?"
             " AND m.import_type = ?";
     if ( params == nullptr || params->includeMissing == false )
@@ -2221,7 +2220,6 @@ Query<IMedia> Media::search( MediaLibraryPtr ml, const std::string& title,
     return make_query<Media, IMedia>( ml, "m.*", std::move( req ),
                                       sortRequest( params ),
                                       sqlite::Tools::sanitizePattern( title ),
-                                      File::Type::Main, File::Type::Disc,
                                       type, ImportType::Internal );
 }
 
@@ -2229,59 +2227,56 @@ Query<IMedia> Media::searchAlbumTracks(MediaLibraryPtr ml, const std::string& pa
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, true );
+    req += addRequestJoin( params );
     req +=  " WHERE"
             " m.id_media IN (SELECT rowid FROM " + Media::FtsTable::Name +
             " WHERE " + Media::FtsTable::Name + " MATCH ?)"
             " AND m.album_id = ?"
-            " AND f.type = ?"
             " AND m.subtype = ?";
     if ( params == nullptr || params->includeMissing == false )
         req += " AND m.is_present != 0";
     return make_query<Media, IMedia>( ml, "m.*", std::move( req ),
                                       sortRequest( params ),
                                       sqlite::Tools::sanitizePattern( pattern ),
-                                      albumId, File::Type::Main, Media::SubType::AlbumTrack );
+                                      albumId, Media::SubType::AlbumTrack );
 }
 
 Query<IMedia> Media::searchArtistTracks(MediaLibraryPtr ml, const std::string& pattern, int64_t artistId, const QueryParameters* params)
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, true );
+    req += addRequestJoin( params );
 
     req +=  " WHERE"
             " m.id_media IN (SELECT rowid FROM " + Media::FtsTable::Name +
             " WHERE " + Media::FtsTable::Name + " MATCH ?)"
             " AND m.artist_id = ?"
-            " AND f.type = ?"
             " AND m.subtype = ?";
     if ( params == nullptr || params->includeMissing == false )
         req += " AND m.is_present != 0";
     return make_query<Media, IMedia>( ml, "m.*", std::move( req ),
                                       sortRequest( params ),
                                       sqlite::Tools::sanitizePattern( pattern ),
-                                      artistId, File::Type::Main, Media::SubType::AlbumTrack );
+                                      artistId, Media::SubType::AlbumTrack );
 }
 
 Query<IMedia> Media::searchGenreTracks(MediaLibraryPtr ml, const std::string& pattern, int64_t genreId, const QueryParameters* params)
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, true );
+    req += addRequestJoin( params );
 
     req +=  " WHERE"
             " m.id_media IN (SELECT rowid FROM " + Media::FtsTable::Name +
             " WHERE " + Media::FtsTable::Name + " MATCH ?)"
             " AND m.genre_id = ?"
-            " AND f.type = ?"
             " AND m.subtype = ?";
     if ( params == nullptr || params->includeMissing == false )
         req += " AND m.is_present != 0";
     return make_query<Media, IMedia>( ml, "m.*", std::move( req ),
                                       sortRequest( params ),
                                       sqlite::Tools::sanitizePattern( pattern ),
-                                      genreId, File::Type::Main, Media::SubType::AlbumTrack );
+                                      genreId, Media::SubType::AlbumTrack );
 }
 
 Query<IMedia> Media::searchShowEpisodes(MediaLibraryPtr ml, const std::string& pattern,
@@ -2289,21 +2284,20 @@ Query<IMedia> Media::searchShowEpisodes(MediaLibraryPtr ml, const std::string& p
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, true );
+    req += addRequestJoin( params );
 
     req +=  " INNER JOIN " + ShowEpisode::Table::Name + " ep ON ep.media_id = m.id_media "
             " WHERE"
             " m.id_media IN (SELECT rowid FROM " + Media::FtsTable::Name +
             " WHERE " + Media::FtsTable::Name + " MATCH ?)"
             " AND ep.show_id = ?"
-            " AND f.type = ?"
             " AND m.subtype = ?";
     if ( params == nullptr || params->includeMissing == false )
         req += " AND m.is_present != 0";
     return make_query<Media, IMedia>( ml, "m.*", std::move( req ),
                                       sortRequest( params ),
                                       sqlite::Tools::sanitizePattern( pattern ),
-                                      showId, File::Type::Main, Media::SubType::ShowEpisode );
+                                      showId, Media::SubType::ShowEpisode );
 }
 
 Query<IMedia> Media::searchInPlaylist( MediaLibraryPtr ml, const std::string& pattern,
@@ -2311,7 +2305,7 @@ Query<IMedia> Media::searchInPlaylist( MediaLibraryPtr ml, const std::string& pa
 {
     std::string req = "FROM " + Media::Table::Name + " m ";
 
-    req += addRequestJoin( params, true );
+    req += addRequestJoin( params );
 
     req += "LEFT JOIN " + Playlist::MediaRelationTable::Name + " pmr "
            "ON pmr.media_id = m.id_media "
@@ -2373,7 +2367,7 @@ Query<IMedia> Media::fromFolderId( MediaLibraryPtr ml, IMedia::Type type,
      * to them.
      */
     std::string req = "FROM " + Table::Name +  " m ";
-    req += addRequestJoin( params, false );
+    req += addRequestJoin( params );
     req += " WHERE m.folder_id = ?";
     if ( type != Type::Unknown )
     {
@@ -2393,7 +2387,7 @@ Query<IMedia> Media::searchFromFolderId( MediaLibraryPtr ml,
                                          const QueryParameters* params )
 {
     std::string req = "FROM " + Table::Name +  " m ";
-    req += addRequestJoin( params, false );
+    req += addRequestJoin( params );
     req += " WHERE m.folder_id = ?";
     req += " AND m.id_media IN (SELECT rowid FROM " + Media::FtsTable::Name +
     " WHERE " + Media::FtsTable::Name + " MATCH ?)";
@@ -2401,7 +2395,9 @@ Query<IMedia> Media::searchFromFolderId( MediaLibraryPtr ml,
     {
         req += " AND m.type = ?";
         return make_query<Media, IMedia>( ml, "*", req, sortRequest( params ),
-                                          folderId, pattern, type );
+                                          folderId,
+                                          sqlite::Tools::sanitizePattern( pattern ),
+                                          type );
     }
     // Don't explicitely filter by type since only video/audio media have a
     // non NULL folder_id
@@ -2414,7 +2410,7 @@ Query<IMedia> Media::fromMediaGroup(MediaLibraryPtr ml, int64_t groupId, Type ty
                                      const QueryParameters* params )
 {
     std::string req = "FROM " + Table::Name + " m ";
-    req += addRequestJoin( params, false );
+    req += addRequestJoin( params );
     req += " WHERE m.group_id = ? AND m.import_type = ?";
     if ( params == nullptr || params->includeMissing == false )
         req += " AND m.is_present != 0";
@@ -2433,10 +2429,8 @@ Query<IMedia> Media::searchFromMediaGroup( MediaLibraryPtr ml, int64_t groupId,
                                            const std::string& pattern,
                                            const QueryParameters* params )
 {
-    if ( pattern.size() < 3 )
-        return nullptr;
     std::string req = "FROM " + Table::Name + " m ";
-    req += addRequestJoin( params, false );
+    req += addRequestJoin( params );
     req += " WHERE m.id_media IN (SELECT rowid FROM " + FtsTable::Name +
             " WHERE " + FtsTable::Name + " MATCH ?)"
            " AND m.group_id = ? AND m.import_type = ?";
