@@ -75,7 +75,11 @@ Status VLCMetadataService::run( IItem& item )
 
     // Having a valid media means we're re-executing this parser after the thumbnailer,
     // which isn't expected, as we always mark this task as completed.
+#if LIBVLC_VERSION_INT >= LIBVLC_VERSION(4, 0, 0, 0)
+    VLC::Media vlcMedia{ mrl, VLC::Media::FromType::FromLocation };
+#else
     VLC::Media vlcMedia{ VLCInstance::get(), mrl, VLC::Media::FromType::FromLocation };
+#endif
 
     VLC::Media::ParsedStatus status;
     bool done = false;
@@ -128,8 +132,14 @@ Status VLCMetadataService::run( IItem& item )
             m_currentMedia = vlcMedia;
         }
 
+#if LIBVLC_VERSION_INT >= LIBVLC_VERSION(4, 0, 0, 0)
+        if ( vlcMedia.parseRequest( VLCInstance::get(),
+                                    VLC::Media::ParseFlags::Local |
+                                    VLC::Media::ParseFlags::Network, 5000 ) == false )
+#else
         if ( vlcMedia.parseWithOptions( VLC::Media::ParseFlags::Local |
                                         VLC::Media::ParseFlags::Network, 5000 ) == false )
+#endif
         {
             std::unique_lock<compat::Mutex> lock( m_mutex );
             m_currentMedia = VLC::Media{};
@@ -191,7 +201,13 @@ void VLCMetadataService::stop()
 {
     std::lock_guard<compat::Mutex> lock{ m_mutex };
     if ( m_currentMedia.isValid() )
+    {
+#if LIBVLC_VERSION_INT >= LIBVLC_VERSION(4, 0, 0, 0)
+        m_currentMedia.parseStop( VLCInstance::get() );
+#else
         m_currentMedia.parseStop();
+#endif
+    }
 }
 
 void VLCMetadataService::mediaToItem( VLC::Media& media, IItem& item )
