@@ -83,11 +83,6 @@ void ModificationNotifier::notifyMediaRemoval( int64_t mediaId )
     notifyRemoval( mediaId, m_media );
 }
 
-void ModificationNotifier::notifyMediaConvertedToExternal( int64_t mediaId )
-{
-    notifyRemoval( mediaId, m_convertedMedia );
-}
-
 void ModificationNotifier::notifyArtistCreation( ArtistPtr artist )
 {
     notifyCreation( std::move( artist ), m_artists );
@@ -229,7 +224,6 @@ void ModificationNotifier::run()
     Queue<IBookmark> bookmarks;
     Queue<IFolder> folders;
     Queue<void> thumbnailsCleanup;
-    Queue<void> convertedMedia;
 
     TimeoutChrono timeout = ZeroTimeout;
 
@@ -286,7 +280,6 @@ void ModificationNotifier::run()
             checkQueue( m_mediaGroups, mediaGroups, nextTimeout, now, flushing );
             checkQueue( m_thumbnailsCleanupRequests, thumbnailsCleanup, nextTimeout, now, flushing );
             checkQueue( m_bookmarks, bookmarks, nextTimeout, now, flushing );
-            checkQueue( m_convertedMedia, convertedMedia, nextTimeout, now, flushing );
             checkQueue( m_folders, folders, nextTimeout, now, flushing );
             timeout = nextTimeout;
 
@@ -304,7 +297,6 @@ void ModificationNotifier::run()
                     &IMediaLibraryCb::onMediaGroupsModified, &IMediaLibraryCb::onMediaGroupsDeleted );
             notify( std::move( bookmarks ), &IMediaLibraryCb::onBookmarksAdded,
                     &IMediaLibraryCb::onBookmarksModified, &IMediaLibraryCb::onBookmarksDeleted );
-            notify( std::move( convertedMedia ), &IMediaLibraryCb::onMediaConvertedToExternal );
             notify( std::move( folders ), &IMediaLibraryCb::onFoldersAdded,
                     &IMediaLibraryCb::onFoldersModified, &IMediaLibraryCb::onFoldersDeleted );
 
